@@ -1,32 +1,27 @@
-#
-# -*- coding: utf-8 -*-
-"""Statement parsing classes for cmd2"""
+"""Statement parsing classes for cmd2."""
 
 import re
 import shlex
-from typing import (
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Tuple,
-    Union,
+from collections.abc import Iterable
+from dataclasses import (
+    dataclass,
+    field,
 )
-
-import attr
+from typing import Any
 
 from . import (
     constants,
     utils,
 )
-from .exceptions import (
-    Cmd2ShlexError,
-)
+from . import string_utils as su
+from .exceptions import Cmd2ShlexError
 
 
-def shlex_split(str_to_split: str) -> List[str]:
-    """
+def shlex_split(str_to_split: str) -> list[str]:
+    """Split the string *str_to_split* using shell-like syntax.
+
     A wrapper around shlex.split() that uses cmd2's preferred arguments.
+
     This allows other classes to easily call split() the same way StatementParser does.
 
     :param str_to_split: the string being split
@@ -35,23 +30,24 @@ def shlex_split(str_to_split: str) -> List[str]:
     return shlex.split(str_to_split, comments=False, posix=False)
 
 
-@attr.s(frozen=True)
+@dataclass(frozen=True)
 class MacroArg:
-    """
-    Information used to replace or unescape arguments in a macro value when the macro is resolved
+    """Information used to replace or unescape arguments in a macro value when the macro is resolved.
+
     Normal argument syntax:    {5}
-    Escaped argument syntax:  {{5}}
+    Escaped argument syntax:  {{5}}.
     """
+
     # The starting index of this argument in the macro value
-    start_index = attr.ib(validator=attr.validators.instance_of(int))
+    start_index: int
 
     # The number string that appears between the braces
     # This is a string instead of an int because we support unicode digits and must be able
     # to reproduce this string later
-    number_str = attr.ib(validator=attr.validators.instance_of(str))
+    number_str: str
 
     # Tells if this argument is escaped and therefore needs to be unescaped
-    is_escaped = attr.ib(validator=attr.validators.instance_of(bool))
+    is_escaped: bool
 
     # Pattern used to find normal argument
     # Digits surrounded by exactly 1 brace on a side and 1 or more braces on the opposite side
@@ -67,25 +63,25 @@ class MacroArg:
     digit_pattern = re.compile(r'\d+')
 
 
-@attr.s(frozen=True)
+@dataclass(frozen=True)
 class Macro:
-    """Defines a cmd2 macro"""
+    """Defines a cmd2 macro."""
 
     # Name of the macro
-    name = attr.ib(validator=attr.validators.instance_of(str))
+    name: str
 
     # The string the macro resolves to
-    value = attr.ib(validator=attr.validators.instance_of(str))
+    value: str
 
     # The minimum number of args the user has to pass to this macro
-    minimum_arg_count = attr.ib(validator=attr.validators.instance_of(int))
+    minimum_arg_count: int
 
     # Used to fill in argument placeholders in the macro
-    arg_list = attr.ib(default=attr.Factory(list), validator=attr.validators.instance_of(list))
+    arg_list: list[MacroArg] = field(default_factory=list)
 
 
-@attr.s(frozen=True)
-class Statement(str):
+@dataclass(frozen=True)
+class Statement(str):  # noqa: SLOT000
     """String subclass with additional attributes to store the results of parsing.
 
     The ``cmd`` module in the standard library passes commands around as a
@@ -94,7 +90,7 @@ class Statement(str):
     we add our own attributes to this subclass.
 
     Instances of this class should not be created by anything other than the
-    :meth:`cmd2.parsing.StatementParser.parse` method, nor should any of the
+    [StatementParser.parse][cmd2.parsing.StatementParser.parse] method, nor should any of the
     attributes be modified once the object is created.
 
     The string portion of the class contains the arguments, but not the
@@ -104,57 +100,60 @@ class Statement(str):
 
     1. `argparse <https://docs.python.org/3/library/argparse.html>`_ is your
        friend for anything complex. ``cmd2`` has the decorator
-       (:func:`~cmd2.decorators.with_argparser`) which you can
+       ([cmd2.decorators.with_argparser][]) which you can
        use to make your command method receive a namespace of parsed arguments,
        whether positional or denoted with switches.
 
     2. For commands with simple positional arguments, use
-       :attr:`~cmd2.Statement.args` or :attr:`~cmd2.Statement.arg_list`
+       [args][cmd2.parsing.Statement.args] or [arg_list][cmd2.parsing.Statement.arg_list]
 
     3. If you don't want to have to worry about quoted arguments, see
-       :attr:`argv` for a trick which strips quotes off for you.
+       [argv][cmd2.parsing.Statement.argv] for a trick which strips quotes off for you.
     """
+
     # the arguments, but not the command, nor the output redirection clauses.
-    args = attr.ib(default='', validator=attr.validators.instance_of(str))
+    args: str = ''
 
     # string containing exactly what we input by the user
-    raw = attr.ib(default='', validator=attr.validators.instance_of(str))
+    raw: str = ''
 
     # the command, i.e. the first whitespace delimited word
-    command = attr.ib(default='', validator=attr.validators.instance_of(str))
+    command: str = ''
 
     # list of arguments to the command, not including any output redirection or terminators; quoted args remain quoted
-    arg_list = attr.ib(default=attr.Factory(list), validator=attr.validators.instance_of(list))
+    arg_list: list[str] = field(default_factory=list)
 
     # if the command is a multiline command, the name of the command, otherwise empty
-    multiline_command = attr.ib(default='', validator=attr.validators.instance_of(str))
+    multiline_command: str = ''
 
     # the character which terminated the multiline command, if there was one
-    terminator = attr.ib(default='', validator=attr.validators.instance_of(str))
+    terminator: str = ''
 
     # characters appearing after the terminator but before output redirection, if any
-    suffix = attr.ib(default='', validator=attr.validators.instance_of(str))
+    suffix: str = ''
 
     # if output was piped to a shell command, the shell command as a string
-    pipe_to = attr.ib(default='', validator=attr.validators.instance_of(str))
+    pipe_to: str = ''
 
     # if output was redirected, the redirection token, i.e. '>>'
-    output = attr.ib(default='', validator=attr.validators.instance_of(str))
+    output: str = ''
 
     # if output was redirected, the destination file token (quotes preserved)
-    output_to = attr.ib(default='', validator=attr.validators.instance_of(str))
+    output_to: str = ''
 
-    def __new__(cls, value: object, *pos_args, **kw_args):
+    # Used in JSON dictionaries
+    _args_field = 'args'
+
+    def __new__(cls, value: object, *_pos_args: Any, **_kw_args: Any) -> 'Statement':
         """Create a new instance of Statement.
 
         We must override __new__ because we are subclassing `str` which is
         immutable and takes a different number of arguments as Statement.
 
-        NOTE:  attrs takes care of initializing other members in the __init__ it
+        NOTE:  @dataclass takes care of initializing other members in the __init__ it
         generates.
         """
-        stmt = super().__new__(cls, value)
-        return stmt
+        return super().__new__(cls, value)
 
     @property
     def command_and_args(self) -> str:
@@ -164,7 +163,7 @@ class Statement(str):
         excluded, as are any command terminators.
         """
         if self.command and self.args:
-            rtn = '{} {}'.format(self.command, self.args)
+            rtn = f'{self.command} {self.args}'
         elif self.command:
             # there were no arguments to the command
             rtn = self.command
@@ -174,7 +173,7 @@ class Statement(str):
 
     @property
     def post_command(self) -> str:
-        """A string containing any ending terminator, suffix, and redirection chars"""
+        """A string containing any ending terminator, suffix, and redirection chars."""
         rtn = ''
         if self.terminator:
             rtn += self.terminator
@@ -194,13 +193,12 @@ class Statement(str):
 
     @property
     def expanded_command_line(self) -> str:
-        """Concatenate :meth:`~cmd2.Statement.command_and_args`
-        and :meth:`~cmd2.Statement.post_command`"""
+        """Concatenate [cmd2.parsing.Statement.command_and_args]() and [cmd2.parsing.Statement.post_command]()."""
         return self.command_and_args + self.post_command
 
     @property
-    def argv(self) -> List[str]:
-        """a list of arguments a-la ``sys.argv``.
+    def argv(self) -> list[str]:
+        """A list of arguments a-la ``sys.argv``.
 
         The first element of the list is the command after shortcut and macro
         expansion. Subsequent elements of the list contain any additional
@@ -210,22 +208,48 @@ class Statement(str):
         If you want to strip quotes from the input, you can use ``argv[1:]``.
         """
         if self.command:
-            rtn = [utils.strip_quotes(self.command)]
-            for cur_token in self.arg_list:
-                rtn.append(utils.strip_quotes(cur_token))
+            rtn = [su.strip_quotes(self.command)]
+            rtn.extend(su.strip_quotes(cur_token) for cur_token in self.arg_list)
         else:
             rtn = []
 
         return rtn
 
+    def to_dict(self) -> dict[str, Any]:
+        """Convert this Statement into a dictionary for use in persistent JSON history files."""
+        return self.__dict__.copy()
+
+    @staticmethod
+    def from_dict(source_dict: dict[str, Any]) -> 'Statement':
+        """Restore a Statement from a dictionary.
+
+        :param source_dict: source data dictionary (generated using to_dict())
+        :return: Statement object
+        :raises KeyError: if source_dict is missing required elements
+        """
+        # value needs to be passed as a positional argument. It corresponds to the args field.
+        try:
+            value = source_dict[Statement._args_field]
+        except KeyError as ex:
+            raise KeyError(f"Statement dictionary is missing {ex} field") from None
+
+        # Pass the rest at kwargs (minus args)
+        kwargs = source_dict.copy()
+        del kwargs[Statement._args_field]
+
+        return Statement(value, **kwargs)
+
 
 class StatementParser:
     """Parse user input as a string into discrete command components."""
-    def __init__(self,
-                 terminators: Optional[Iterable[str]] = None,
-                 multiline_commands: Optional[Iterable[str]] = None,
-                 aliases: Optional[Dict[str, str]] = None,
-                 shortcuts: Optional[Dict[str, str]] = None) -> None:
+
+    def __init__(
+        self,
+        terminators: Iterable[str] | None = None,
+        multiline_commands: Iterable[str] | None = None,
+        aliases: dict[str, str] | None = None,
+        shortcuts: dict[str, str] | None = None,
+    ) -> None:
         """Initialize an instance of StatementParser.
 
         The following will get converted to an immutable tuple before storing internally:
@@ -236,18 +260,13 @@ class StatementParser:
         :param aliases: dictionary containing aliases
         :param shortcuts: dictionary containing shortcuts
         """
+        self.terminators: tuple[str, ...]
         if terminators is None:
             self.terminators = (constants.MULTILINE_TERMINATOR,)
         else:
             self.terminators = tuple(terminators)
-        if multiline_commands is None:
-            self.multiline_commands = tuple()
-        else:
-            self.multiline_commands = tuple(multiline_commands)
-        if aliases is None:
-            self.aliases = dict()
-        else:
-            self.aliases = aliases
+        self.multiline_commands: tuple[str, ...] = tuple(multiline_commands) if multiline_commands is not None else ()
+        self.aliases: dict[str, str] = aliases if aliases is not None else {}
 
         if shortcuts is None:
             shortcuts = constants.DEFAULT_SHORTCUTS
@@ -285,13 +304,13 @@ class StatementParser:
         # join them up with a pipe
         second_group = '|'.join(second_group_items)
         # build the regular expression
-        expr = r'\A\s*(\S*?)({})'.format(second_group)
+        expr = rf'\A\s*(\S*?)({second_group})'
         self._command_pattern = re.compile(expr)
 
-    def is_valid_command(self, word: str, *, is_subcommand: bool = False) -> Tuple[bool, str]:
+    def is_valid_command(self, word: str, *, is_subcommand: bool = False) -> tuple[bool, str]:
         """Determine whether a word is a valid name for a command.
 
-        Commands can not include redirection characters, whitespace,
+        Commands cannot include redirection characters, whitespace,
         or termination characters. They also cannot start with a
         shortcut.
 
@@ -305,12 +324,12 @@ class StatementParser:
             checkit = '>'
             valid, errmsg = statement_parser.is_valid_command(checkit)
             if not valid:
-                errmsg = "alias: {}".format(errmsg)
+                errmsg = f"alias: {errmsg}"
         """
         valid = False
 
         if not isinstance(word, str):
-            return False, 'must be a string. Received {} instead'.format(str(type(word)))
+            return False, f'must be a string. Received {type(word)!s} instead'  # type: ignore[unreachable]
 
         if not word:
             return False, 'cannot be an empty string'
@@ -319,7 +338,7 @@ class StatementParser:
             return False, 'cannot start with the comment character'
 
         if not is_subcommand:
-            for (shortcut, _) in self.shortcuts:
+            for shortcut, _ in self.shortcuts:
                 if word.startswith(shortcut):
                     # Build an error string with all shortcuts listed
                     errmsg = 'cannot start with a shortcut: '
@@ -333,22 +352,18 @@ class StatementParser:
         errmsg += ', '.join([shlex.quote(x) for x in errchars])
 
         match = self._command_pattern.search(word)
-        if match:
-            if word == match.group(1):
-                valid = True
-                errmsg = ''
+        if match and word == match.group(1):
+            valid = True
+            errmsg = ''
         return valid, errmsg
 
-    def tokenize(self, line: str) -> List[str]:
-        """
-        Lex a string into a list of tokens. Shortcuts and aliases are expanded and
-        comments are removed.
+    def tokenize(self, line: str) -> list[str]:
+        """Lex a string into a list of tokens. Shortcuts and aliases are expanded and comments are removed.
 
         :param line: the command line being lexed
         :return: A list of tokens
-        :raises: Cmd2ShlexError if a shlex error occurs (e.g. No closing quotation)
+        :raises Cmd2ShlexError: if a shlex error occurs (e.g. No closing quotation)
         """
-
         # expand shortcuts and aliases
         line = self._expand(line)
 
@@ -360,23 +375,20 @@ class StatementParser:
         try:
             tokens = shlex_split(line)
         except ValueError as ex:
-            raise Cmd2ShlexError(ex)
+            raise Cmd2ShlexError(ex) from None
 
         # custom lexing
-        tokens = self.split_on_punctuation(tokens)
-        return tokens
+        return self.split_on_punctuation(tokens)
 
     def parse(self, line: str) -> Statement:
-        """
-        Tokenize the input and parse it into a :class:`~cmd2.Statement` object,
-        stripping comments, expanding aliases and shortcuts, and extracting output
-        redirection directives.
+        """Tokenize the input and parse it into a [cmd2.parsing.Statement][] object.
+
+        Stripping comments, expanding aliases and shortcuts, and extracting output redirection directives.
 
         :param line: the command line being parsed
-        :return: a new :class:`~cmd2.Statement` object
-        :raises: Cmd2ShlexError if a shlex error occurs (e.g. No closing quotation)
+        :return: a new [cmd2.parsing.Statement][] object
+        :raises Cmd2ShlexError: if a shlex error occurs (e.g. No closing quotation)
         """
-
         # handle the special case/hardcoded terminator of a blank line
         # we have to do this before we tokenize because tokenizing
         # destroys all unquoted whitespace in the input
@@ -418,7 +430,7 @@ class StatementParser:
             arg_list = tokens[1:terminator_pos]
             # we will set the suffix later
             # remove all the tokens before and including the terminator
-            tokens = tokens[terminator_pos + 1:]
+            tokens = tokens[terminator_pos + 1 :]
         else:
             (testcommand, testargs) = self._command_and_args(tokens)
             if testcommand in self.multiline_commands:
@@ -452,9 +464,8 @@ class StatementParser:
 
         # Check if output should be piped to a shell command
         if pipe_index < redir_index and pipe_index < append_index:
-
             # Get the tokens for the pipe command and expand ~ where needed
-            pipe_to_tokens = tokens[pipe_index + 1:]
+            pipe_to_tokens = tokens[pipe_index + 1 :]
             utils.expand_user_in_tokens(pipe_to_tokens)
 
             # Build the pipe command line string
@@ -474,7 +485,7 @@ class StatementParser:
 
             # Check if we are redirecting to a file
             if len(tokens) > output_index + 1:
-                unquoted_path = utils.strip_quotes(tokens[output_index + 1])
+                unquoted_path = su.strip_quotes(tokens[output_index + 1])
                 if unquoted_path:
                     output_to = utils.expand_user(tokens[output_index + 1])
 
@@ -493,26 +504,24 @@ class StatementParser:
                 arg_list = tokens[1:]
 
         # set multiline
-        if command in self.multiline_commands:
-            multiline_command = command
-        else:
-            multiline_command = ''
+        multiline_command = command if command in self.multiline_commands else ''
 
         # build the statement
-        statement = Statement(args,
-                              raw=line,
-                              command=command,
-                              arg_list=arg_list,
-                              multiline_command=multiline_command,
-                              terminator=terminator,
-                              suffix=suffix,
-                              pipe_to=pipe_to,
-                              output=output,
-                              output_to=output_to)
-        return statement
+        return Statement(
+            args,
+            raw=line,
+            command=command,
+            arg_list=arg_list,
+            multiline_command=multiline_command,
+            terminator=terminator,
+            suffix=suffix,
+            pipe_to=pipe_to,
+            output=output,
+            output_to=output_to,
+        )
 
     def parse_command_only(self, rawinput: str) -> Statement:
-        """Partially parse input into a :class:`~cmd2.Statement` object.
+        """Parse input into a [cmd2.Statement][] object (partially).
 
         The command is identified, and shortcuts and aliases are expanded.
         Multiline commands are identified, but terminators and output
@@ -521,21 +530,21 @@ class StatementParser:
         This method is used by tab completion code and therefore must not
         generate an exception if there are unclosed quotes.
 
-        The :class:`~cmd2.Statement` object returned by this method can at most
+        The [cmd2.parsing.Statement][] object returned by this method can at most
         contain values in the following attributes:
-        :attr:`~cmd2.Statement.args`, :attr:`~cmd2.Statement.raw`,
-        :attr:`~cmd2.Statement.command`,
-        :attr:`~cmd2.Statement.multiline_command`
+        [cmd2.parsing.Statement.args][], [cmd2.parsing.Statement.raw][],
+        [cmd2.parsing.Statement.command][],
+        [cmd2.parsing.Statement.multiline_command][]
 
-        :attr:`~cmd2.Statement.args` will include all output redirection
+        [cmd2.parsing.Statement.args][] will include all output redirection
         clauses and command terminators.
 
-        Different from :meth:`~cmd2.parsing.StatementParser.parse` this method
+        Different from [cmd2.parsing.StatementParser.parse][] this method
         does not remove redundant whitespace within args. However, it does
         ensure args has no leading or trailing whitespace.
 
         :param rawinput: the command line as entered by the user
-        :return: a new :class:`~cmd2.Statement` object
+        :return: a new [cmd2.Statement][] object
         """
         # expand shortcuts and aliases
         line = self._expand(rawinput)
@@ -549,8 +558,15 @@ class StatementParser:
 
             # take everything from the end of the first match group to
             # the end of the line as the arguments (stripping leading
-            # and trailing spaces)
-            args = line[match.end(1):].strip()
+            # and unquoted trailing whitespace)
+            args = line[match.end(1) :].lstrip()
+            try:
+                shlex_split(args)
+            except ValueError:
+                # Unclosed quote. Leave trailing whitespace.
+                pass
+            else:
+                args = args.rstrip()
             # if the command is empty that means the input was either empty
             # or something weird like '>'. args should be empty if we couldn't
             # parse a command
@@ -558,40 +574,33 @@ class StatementParser:
                 args = ''
 
         # set multiline
-        if command in self.multiline_commands:
-            multiline_command = command
-        else:
-            multiline_command = ''
+        multiline_command = command if command in self.multiline_commands else ''
 
         # build the statement
-        statement = Statement(args,
-                              raw=rawinput,
-                              command=command,
-                              multiline_command=multiline_command)
-        return statement
+        return Statement(args, raw=rawinput, command=command, multiline_command=multiline_command)
+
+    def get_command_arg_list(
+        self, command_name: str, to_parse: Statement | str, preserve_quotes: bool
+    ) -> tuple[Statement, list[str]]:
+        """Retrieve just the arguments being passed to their ``do_*`` methods as a list.
 
-    def get_command_arg_list(self, command_name: str, to_parse: Union[Statement, str],
-                             preserve_quotes: bool) -> Tuple[Statement, List[str]]:
-        """
         Convenience method used by the argument parsing decorators.
 
-        Retrieves just the arguments being passed to their ``do_*`` methods as a list.
-
         :param command_name: name of the command being run
         :param to_parse: what is being passed to the ``do_*`` method. It can be one of two types:
 
-                             1. An already parsed :class:`~cmd2.Statement`
+                             1. An already parsed [cmd2.Statement][]
                              2. An argument string in cases where a ``do_*`` method is
                                 explicitly called. Calling ``do_help('alias create')`` would
                                 cause ``to_parse`` to be 'alias create'.
 
                                 In this case, the string will be converted to a
-                                :class:`~cmd2.Statement` and returned along with
+                                [cmd2.Statement][] and returned along with
                                 the argument list.
 
         :param preserve_quotes: if ``True``, then quotes will not be stripped from
                                 the arguments
-        :return: A tuple containing the :class:`~cmd2.Statement` and a list of
+        :return: A tuple containing the [cmd2.Statement][] and a list of
                  strings representing the arguments
         """
         # Check if to_parse needs to be converted to a Statement
@@ -600,12 +609,10 @@ class StatementParser:
 
         if preserve_quotes:
             return to_parse, to_parse.arg_list
-        else:
-            return to_parse, to_parse.argv[1:]
+        return to_parse, to_parse.argv[1:]
 
     def _expand(self, line: str) -> str:
-        """Expand aliases and shortcuts"""
-
+        """Expand aliases and shortcuts."""
         # Make a copy of aliases so we can keep track of what aliases have been resolved to avoid an infinite loop
         remaining_aliases = list(self.aliases.keys())
         keep_expanding = bool(remaining_aliases)
@@ -622,28 +629,27 @@ class StatementParser:
                 # Check if this command matches an alias that wasn't already processed
                 if command in remaining_aliases:
                     # rebuild line with the expanded alias
-                    line = self.aliases[command] + match.group(2) + line[match.end(2):]
+                    line = self.aliases[command] + match.group(2) + line[match.end(2) :]
                     remaining_aliases.remove(command)
                     keep_expanding = bool(remaining_aliases)
 
         # expand shortcuts
-        for (shortcut, expansion) in self.shortcuts:
+        for shortcut, expansion in self.shortcuts:
             if line.startswith(shortcut):
                 # If the next character after the shortcut isn't a space, then insert one
                 shortcut_len = len(shortcut)
+                effective_expansion = expansion
                 if len(line) == shortcut_len or line[shortcut_len] != ' ':
-                    expansion += ' '
+                    effective_expansion += ' '
 
                 # Expand the shortcut
-                line = line.replace(shortcut, expansion, 1)
+                line = line.replace(shortcut, effective_expansion, 1)
                 break
         return line
 
     @staticmethod
-    def _command_and_args(tokens: List[str]) -> Tuple[str, str]:
-        """Given a list of tokens, return a tuple of the command
-        and the args as a string.
-        """
+    def _command_and_args(tokens: list[str]) -> tuple[str, str]:
+        """Given a list of tokens, return a tuple of the command and the args as a string."""
         command = ''
         args = ''
 
@@ -655,7 +661,7 @@ class StatementParser:
 
         return command, args
 
-    def split_on_punctuation(self, tokens: List[str]) -> List[str]:
+    def split_on_punctuation(self, tokens: list[str]) -> list[str]:
         """Further splits tokens from a command line using punctuation characters.
 
         Punctuation characters are treated as word breaks when they are in
@@ -665,14 +671,13 @@ class StatementParser:
         :param tokens: the tokens as parsed by shlex
         :return: a new list of tokens, further split using punctuation
         """
-        punctuation = []
+        punctuation: list[str] = []
         punctuation.extend(self.terminators)
         punctuation.extend(constants.REDIRECTION_CHARS)
 
         punctuated_tokens = []
 
         for cur_initial_token in tokens:
-
             # Save tokens up to 1 character in length or quoted tokens. No need to parse these.
             if len(cur_initial_token) <= 1 or cur_initial_token[0] in constants.QUOTES:
                 punctuated_tokens.append(cur_initial_token)
@@ -687,7 +692,6 @@ class StatementParser:
 
             while True:
                 if cur_char not in punctuation:
-
                     # Keep appending to new_token until we hit a punctuation char
                     while cur_char not in punctuation:
                         new_token += cur_char
