@@ -12,21 +12,16 @@ import pathlib
 import re
 import shutil
 import sys
-from typing import Union
 
 import invoke
 from invoke.context import Context
 
-from plugins import (
-    tasks as plugin_tasks,
-)
-
 TASK_ROOT = pathlib.Path(__file__).resolve().parent
 TASK_ROOT_STR = str(TASK_ROOT)
 
 
 # shared function
-def rmrf(items: Union[str, list[str], set[str]], verbose: bool = True) -> None:
+def rmrf(items: str | list[str] | set[str], verbose: bool = True) -> None:
     """Silently remove a list of directories or files."""
     if isinstance(items, str):
         items = [items]
@@ -41,7 +36,7 @@ def rmrf(items: Union[str, list[str], set[str]], verbose: bool = True) -> None:
 
 
 # create namespaces
-namespace = invoke.Collection(plugin=plugin_tasks)
+namespace = invoke.Collection()
 namespace_clean = invoke.Collection('clean')
 namespace.add_collection(namespace_clean, 'clean')
 
@@ -53,34 +48,24 @@ namespace.add_collection(namespace_clean, 'clean')
 
 
 @invoke.task()
-def pytest(context: Context, junit: bool = False, pty: bool = True, base: bool = False, isolated: bool = False) -> None:
+def pytest(context: Context, junit: bool = False, pty: bool = True) -> None:
     """Run tests and code coverage using pytest."""
     with context.cd(TASK_ROOT_STR):
         command_str = 'pytest '
         command_str += ' --cov=cmd2 '
         command_str += ' --cov-append --cov-report=term --cov-report=html '
 
-        if not base and not isolated:
-            base = True
-            isolated = True
-
         if junit:
             command_str += ' --junitxml=junit/test-results.xml '
 
-        if base:
-            tests_cmd = command_str + ' tests'
-            context.run(tests_cmd, pty=pty)
-        if isolated:
-            for _root, dirnames, _ in os.walk(str(TASK_ROOT / 'tests_isolated')):
-                for dir_name in dirnames:
-                    if dir_name.startswith('test_'):
-                        context.run(command_str + ' tests_isolated/' + dir_name)
+        tests_cmd = command_str + ' tests'
+        context.run(tests_cmd, pty=pty)
 
 
 namespace.add_task(pytest)
 
 
-@invoke.task(post=[plugin_tasks.pytest_clean])
+@invoke.task()
 def pytest_clean(context: Context) -> None:
     """Remove pytest cache and code coverage files and directories."""
     # pylint: disable=unused-argument
@@ -164,7 +149,7 @@ BUILDDIR = 'build'
 DISTDIR = 'dist'
 
 
-@invoke.task(post=[plugin_tasks.build_clean])
+@invoke.task()
 def build_clean(context: Context) -> None:
     """Remove the build directory."""
     # pylint: disable=unused-argument
@@ -175,7 +160,7 @@ def build_clean(context: Context) -> None:
 namespace_clean.add_task(build_clean, 'build')
 
 
-@invoke.task(post=[plugin_tasks.dist_clean])
+@invoke.task()
 def dist_clean(context: Context) -> None:
     """Remove the dist directory."""
     # pylint: disable=unused-argument
@@ -254,7 +239,6 @@ namespace_clean.add_task(ruff_clean, 'ruff')
 #
 # make a dummy clean task which runs all the tasks in the clean namespace
 clean_tasks = list(namespace_clean.tasks.values())
-clean_tasks.append(plugin_tasks.clean_all)
 
 
 @invoke.task(pre=clean_tasks, default=True)
@@ -286,9 +270,9 @@ def validatetag(context: Context) -> None:
     git_tag = result.stdout.rstrip()
 
     # Validate that the Git tag appears to be a valid version number
-    ver_regex = re.compile(r'(\d+)\.(\d+)\.(\d+)')
-    match = ver_regex.fullmatch(git_tag)
-    if match is None:
+    ver_regex = re.compile(r'(\d+)\.(\d+)\.(\d+)((a|b|rc)(\d+))?')
+    found = ver_regex.fullmatch(git_tag)
+    if found is None:
         print(f'Tag {git_tag!r} does not appear to be a valid version number')
         sys.exit(-1)
     else:
@@ -298,7 +282,7 @@ def validatetag(context: Context) -> None:
 namespace.add_task(validatetag)
 
 
-@invoke.task(pre=[clean_all], post=[plugin_tasks.sdist])
+@invoke.task(pre=[clean_all])
 def sdist(context: Context) -> None:
     """Create a source distribution."""
     with context.cd(TASK_ROOT_STR):
@@ -308,7 +292,7 @@ def sdist(context: Context) -> None:
 namespace.add_task(sdist)
 
 
-@invoke.task(pre=[clean_all], post=[plugin_tasks.wheel])
+@invoke.task(pre=[clean_all])
 def wheel(context: Context) -> None:
     """Build a wheel distribution."""
     with context.cd(TASK_ROOT_STR):
